/* 
 * @(#)ControlPanel.java
 */

package galton.analysis;

import java.awt.*;
import java.awt.event.*;
import java.text.*;
import java.applet.AppletContext;
import galton.analysis.*;
import galton.common.*;

/**
 * A Panel with buttons and labels and textfields in a column.
 *
 */
public class ControlPanel extends Panel
    implements AdjustmentListener, ActionListener, MouseListener {

    private static final int DEBUG = 1;
     
    //**************** constructor ********************//
    public ControlPanel (Language lang, Machine machine, 
                         ActionListener l1, AdjustmentListener l2) {
	this.machine = machine;
      this.lang = lang;

	GridBagLayout gridbag = new GridBagLayout ();
	GridBagConstraints c = new GridBagConstraints ();
	setLayout (gridbag);
	setFont(lang.font_normal);

	c.fill = GridBagConstraints.BOTH;
	c.weightx = 1.0;
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.insets = new Insets (1, 2, 1, 2); //responsible for button padding

	pBar = new Scrollbar (0, lang.getValue(Language.pEXP), 5, 0, 100 + 5);
	pLabel = new Label ();
	//            pLabel.addMouseListener(this);
	pBar.addAdjustmentListener (this);
	pBar.addAdjustmentListener (l2);
      pBar.setValue(lang.getValue(Language.pEXP));
	gridbag.setConstraints (pBar, c);
	gridbag.setConstraints (pLabel, c);
	add (pLabel);
	add (pBar);

	//NumberFormat nf=NumberFormat.getInstance();
	serButton = new Button (lang.getName(Language.SERIE) + " : 0");
	serButton.addActionListener (l1);
	serButton.addActionListener (this);
	serButton.setFont (lang.font_bigger);
	gridbag.setConstraints (serButton, c);
	add (serButton);
	sumButton = new Button (lang.getName(Language.SUM) + " : 0");
	sumButton.addActionListener (l1);
	sumButton.setFont (lang.font_bigger);
	gridbag.setConstraints (sumButton, c);
	add (sumButton);

	flipperSS = new ButtonFlipper (lang.getName(Language.START),lang.getName(Language.STOP));
	flipperSS.addActionListener (l1);
	flipperSS.addActionListener (this);
	flipperSS.setForeground (Color.red);
	gridbag.setConstraints (flipperSS, c);
	add (flipperSS);

	c.insets = new Insets (1, 2, 0, 3);
	fieldn = new NumberField (lang.getName(Language.nSTRING) + "=", 
					  lang.getValue(Language.nMIN),lang.getValue(Language.nMAX));
	fieldN = new NumberField (lang.getName(Language.NSTRING) + "=", 
					  lang.getValue(Language.NMIN),lang.getValue(Language.NMAX));
	fieldn.addActionListener (this);
	fieldn.addActionListener (l1);
	fieldN.addActionListener (this);
	fieldN.addActionListener (l1);
	fieldn.setValue (lang.getValue(Language.nEXP));	// fieldn.parse();
	fieldN.setValue (lang.getValue(Language.NEXP));	// fieldn.parse();

	gridbag.setConstraints (fieldN, c); add (fieldN);
	gridbag.setConstraints (fieldn, c); add (fieldn);

	c.insets = new Insets (0, 2, 0, 0);
	infoLabel = new Label[5];
	for (int i = 0; i < 5; i++) {
	    infoLabel[i] = new Label (makeWide (lang.getName(Language.INFO+i) + " : ", "-", 12), Label.LEFT);
	    infoLabel[i].setFont (lang.font_info);
	    //              infoLabel[i].addMouseListener(this);
	    gridbag.setConstraints (infoLabel[i], c);
	    add (infoLabel[i]);
	}
	c.insets = new Insets (0, 1, 0, 0);
	c.gridwidth = GridBagConstraints.REMAINDER;
	makebutton (lang.getName(Language.EBAR), gridbag, c, this, l1);
	makebutton (lang.getName(Language.TEST), gridbag, c, this, l1);
	c.insets = new Insets (2, 1, 0, 0);
	makebutton (lang.getName(Language.OPTION), gridbag, c, this, l1);
	c.insets = new Insets (0, 1, 0, 0);
	makebutton (lang.getName(Language.RESULT), gridbag, c, this, l1);
	makebutton (lang.getName(Language.HELP), gridbag, c, this, l1);

	this.addMouseListener (this);
	
      for (int i = 0; i < getComponentCount (); i++)
	  getComponent (i).addMouseListener (this);
    }
    public void dispatch () {
	if(DEBUG>=2) System.out.println (""+machine.getSum()+": ControlPanel.dispatch");
	fieldn.parse (true);
	fieldN.parse (true);
	pBar.dispatchEvent (new AdjustmentEvent (pBar, 
						 AdjustmentEvent.ADJUSTMENT_VALUE_CHANGED, 
						 AdjustmentEvent.TRACK, pBar.getValue ()));
    }
    //**************** event handlers *****************//
    public void adjustmentValueChanged (AdjustmentEvent evt) {
	NumberFormat nf = NumberFormat.getInstance ();
	nf.setMaximumFractionDigits (2);
	nf.setMinimumFractionDigits (2);
	String val = nf.format ((double) evt.getAdjustable ().getValue () / 100.0);
	String text = lang.getName(Language.pSTRING).concat(" = ").concat (val);
	if (!pLabel.getText ().equals (text))
	    pLabel.setText (text);
    } 

    public void actionPerformed (ActionEvent evt) {
	if(DEBUG>=2) System.out.println (""+machine.getSum()+": CP.actionPerf:" + evt.getActionCommand ());
	if(DEBUG>=2) System.out.println (""+machine.getSum()+": fieldN:" + evt.getSource ().equals (fieldN.field));

	if (evt.getSource().equals(fieldN.field)) {
	    machine.setN (fieldN.getValue ());
	    serButton.setLabel (lang.getName(Language.SERIE)+" : "+NumberFormat.getInstance().format (machine.getSerie()));
	    return;
	} 
      if (evt.getActionCommand().equals(serButton.getLabel())) {
	    serButton.setLabel (lang.getName(Language.SERIE) + " : 0");
	    return;
	}
	if (evt.getActionCommand().equals(lang.getName(Language.STOP))) {
	    display ();
	    return;
	}
    }

    public void mouseExited (MouseEvent e) {
	lang.getAppletContext().showStatus (Language.DEFAULT_STATUS);
    }
    public void mouseEntered (MouseEvent e) {
	for (int i = 0; i < getComponentCount (); i++)
	    if (getComponent (i).equals (e.getComponent ())) {
		lang.getAppletContext().showStatus (lang.getMessage(i));
		return;
	    } 
	lang.getAppletContext().showStatus(lang.getMessage(Language.MESSAGES-1));
    }
    public void mousePressed (MouseEvent e) {
    }
    public void mouseReleased (MouseEvent e) {
    }
    public void mouseClicked (MouseEvent e) {
    }

    //*************** paint methods *******************//
    public void display () {
	NumberFormat nf = NumberFormat.getInstance ();
	nf.setGroupingUsed (false);
	serButton.setLabel (lang.getName(Language.SERIE) + " : " + nf.format (machine.getSerie ()));
	sumButton.setLabel (lang.getName(Language.SUM) + " : " + nf.format (machine.getSum ()));
	//        sumLabel.setLabel(makeWide(SUM+" : ",""+machine.getSum(),18));
	//        boolean isHuge=machine.getSum()>=(10*NRANGE[2]);
	//        nf.setMaximumIntegerDigits( (int)(Math.log(NRANGE[2])/Math.log(10)));
	//        serieLabel.setText    (SERIE  +" : "  +nf.format(machine.getSerie()) );
	//        sumLabel.setText      (SUM    +" : "  +nf.format(machine.getSum())   );
	if (!machine.isRunning ()) {
	    flipperSS.a ();
	    for (int i = 0; i < this.getComponentCount (); i++)
		this.getComponent (i).setEnabled (true);
	}
	machine.makeInfo ();
	nf.setMaximumFractionDigits (5);
	nf.setMinimumFractionDigits (3);
	for (int i = 0; i < 4; i++)
	    infoLabel[i].setText (makeWide (lang.getName(Language.INFO+i) + " : ", 
					    nf.format (machine.getInfo (i)), 12));
	nf.setMinimumFractionDigits (0);
	infoLabel[4].setText(makeWide (lang.getName(Language.INFO4) + " : ",
                           nf.format (machine.getInfo (4)), 12));
    } 

    Panel panel1, panel2;
    Label pLabel;
    Scrollbar pBar;
    public Button serButton, sumButton;
    public Label[] infoLabel;
    Color color;
    Machine machine;
    NumberField fieldn, fieldN;
    ButtonFlipper flipperSS;

    Language lang;

    private boolean first = true;

    //*************************** methods ***************************//
    protected void makebutton (String name, GridBagLayout gridbag, GridBagConstraints c, 
			       Container comp, ActionListener la) {
	Button button = new Button (name);
	button.addActionListener (la);
	gridbag.setConstraints (button, c);
	comp.add (button);
    }
    protected String trimDouble (double val, int places) {
	String str = "" + val;
	int i = str.indexOf ('.');
	i++;
	boolean useSubstring = i + places <= str.length ();
	if (i == -1)
	    useSubstring = false;
	String res = useSubstring ? str.substring (0, i + places) : str;
	if (useSubstring)
	    return str.substring (0, i + places);
	else
	    return str;
    }
    protected String makeWide (String beg, String end, int length) {
	int i;
	String res = new String (beg);
	for (i = length - beg.length () - end.length (); i > 0; i--)
	    res += " ";
	res += end;
	return res;
    }
}











