/*
 * @(#)Machine.java
 */
package galton.analysis;

import java.util.Random;
/**
 * A Thread used to generate random distributions.
 *
 */
public class Machine extends MachineBase{  

    private Random random;      // pseudorandom number generator
    private boolean running;    //
    private boolean dead=false; //will be used to gracefully exit;

    private boolean AnReadyToWait;

    public int DEBUG=2;

    /**
     * A standard constructor.
     * @param running whether should be running after construction.
     */
    public Machine(boolean running){
        random = new Random();
        this.running=running;
    }

    public boolean isRunning(){
      return running;
    }

    public synchronized void setRunning(boolean running){
	if(DEBUG>=1) System.out.println(""+getSum()+": Mach.setRun("+running+")/beg AnRd2Wait="+AnReadyToWait);
      if (running) AnReadyToWait = false;
      this.running=running;
	if( !isAlive() && running) start();
      notifyAll();
	if(DEBUG>=1) System.out.println(""+getSum()+": Mach.setRun(.)/end AnRd2Wait="+AnReadyToWait);
    }
  
    /**
     * A method called to measure out the time between repaintings.
     * @param time the expected time to wait
     * @param readyToWait if the Object calling has repaint after Machine has stopped running
     */
    public synchronized void getRepaintPermit(int time){
      boolean running=false;
	if(DEBUG>=2) System.out.println(""+getSum()+": Mach.getRepPermit(" +time+") AnRd2Wait="+AnReadyToWait);
	try{
        if( (running=isRunning())==true ){
          wait(time);
        } else {
		if(DEBUG>=2) System.out.println(""+getSum()+" Mach.getRepPermit:readyToWait? "+AnReadyToWait);
		if(AnReadyToWait) {wait(); running=true; }
	  }
        }catch(InterruptedException e){
      }
      if (!running) AnReadyToWait = true;
    }

    /**
     * A method that checks whether Machine.run() method should continue with adding balls.
     * @param finished if the machine should pause (c==N)
     * @return whether Machine should dispose
     */
    protected synchronized boolean getRunPermit(boolean finished){
        if(finished) setRunning(false);
        while(!running){
	    if(DEBUG>=2) System.out.println(""+getSum()+": Mach.getRunPermit:not running, will wait");
          try{
            wait();
          }catch(InterruptedException e){
          }
	    if(DEBUG>=2) System.out.println(""+getSum()+": Mach.getRunPermit:finished waiting");
        }
        if(finished)clearCounter();
        return !dead;   //right now true
    }

    /**
     * Sets n. Automatically pauses Machine.run() by calling setRunning(false) and clears Table, l, max.
     * @see n.
     */
    public synchronized void setn(int n){
	if(DEBUG>=1)System.out.println("Machine.setn( "+n+" )");
        setRunning(false);
        this.n=n;
        Table=new int[n+1][2];
        l=0;
        max=0;
        clearCounter();
        System.gc();
        notifyAll();
    }
    protected synchronized void addToTable(){
        double b=0.0;
        for(int i=0;i<n;i++)
	    if(random.nextFloat()<p) b+=0.5;
	                    else     b-=0.5;
        int k=(int)(b+ktox(n));   // same as xtok(b)
	if(Table[k][VALUE]==max)
            max++;
        Table[k][VALUE]++;
        Table[k][MODIF]++;
        l++;
        c++;
    }
    
    /**
     * To implement Runnable. 
     */
    public void run(){
	if(DEBUG>=2) System.out.println("Mach.run/begin");
      while(getRunPermit(c==N)){
        addToTable();
      }
	if(DEBUG>=2) System.out.println("Mach.run/end");
    }
       
    public synchronized void end(){
	dead=true;
      notifyAll();
    }
}













