/*
 * @(#)MachineBase.java
 */

package galton.analysis;

/**
 * The class containing variables and simple methods for Machine.
 * @see galton.analysis.Machine
 */
public abstract class MachineBase extends Thread{

    /* first the public variables, p,n,N,c,l */

    /**
     * The probability of a ball falling to the left.
     */
    public double p;
    /**
     * Number of decisions. n+1 = the number of columns 0..n.
     */
    public int n;
    /**
     * Number of balls in a serie
     */
    public int N;

    /** Counter : in a serie. */
    protected long c;
    /** Counter : total balls in table. */
    protected long l;

    /**
     * Contains [ticks in column, whether modified since last reading].
     * Table must have the length n+1.
     * see #n
     */
    protected int[][] Table;

    protected static final int MODIF=1, VALUE=0;

    public  int NMAX;

    public int max;             // 
    private double xm,sx,sm,km; // variables describing the distribution

    /**
     * Updates information that can be retrieved through getInfo(). Should be called once
     * before updating all variables retrieved through getInfo().
     * @see #getInfo(int i)
     */
    public synchronized void makeInfo(){
	double _EX=0;
	//first find _EX and xm
	for(int _k=0;_k<=n;_k++)
	    _EX+=(double)Table[_k][VALUE]*ktox(_k);
	xm=_EX/l;
	km=xm+ktox(n);
	//and then other variables
	double _X2=0;
	for(int _k=0;_k<=n;_k++)
	    _X2+=Math.pow(ktox(_k)-xm,2)*Table[_k][VALUE];
	sx=Math.sqrt( _X2/((double)l-1.0));
	sm=Math.sqrt( _X2/(double)l/((double)l-1.0));
    }

    /**
     * @param i 0->xm, 1->km, 2->sx, 3->sm, 4->max
     */
    public double getInfo(int i){
	switch(i){
	case 0 :return xm;
	case 1 :return km;
	case 2 :return sx;
	case 3 :return sm;
	case 4 :return (double)max;
	default :
	    throw new IndexOutOfBoundsException("mach.getInfo : "+i);
	}
    }

    /**
     * Sets p.
     * @param p probability of the ball falling left (between 0.0 and 1.0)
     */
    public void setp(double p){
        this.p=p;
    }

    /**
     * Converts k to x. k is between 0 and n, x is between -n/2 and +n/2.
     */
    public double ktox(double k){
	    return (double)k-(double)n/2.0;
    }
    /**
     * Converts k to x.
     * @see #ktox(double k)
     */
    public double ktox(int k){ return ktox((double)k); }

    public synchronized long getSerie(){ return c; }

    public synchronized long getSum(){ return l; }
   
    /**
     * Sets N. Clears c.
     */
    public void setN(int N){
      clearCounter();
      this.N=N;
    }

    public synchronized void clearCounter(){ c=0; }

    public int getValue(int k){ return Table[k][VALUE]; }

    public int getTop(){
	if(isRunning())
            return (int)( ((double)max+1.0)*1.5 );
	    else
	    return (int)( ((double)max+1.0)*1.2 );
    }

    /**
     * Get a list of racks that have changed.
     */
    public synchronized int[] getModified(){
        int j=0;
        int _length=0;
        for(int k=0;k<=n;k++)
            if(Table[k][MODIF]>0)_length++;
        int[] Modif=new int[_length];
        for(int k=0;k<=n;k++)
            if(Table[k][MODIF]>0){
                Table[k][MODIF]=0;
                Modif[j]=k;
                j++;
            }
        return Modif;
    }

    public abstract void run();
    public abstract boolean isRunning();
}