/*
 * @(#)Analysis.java
 */
package galton.analysis;

import java.applet.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import galton.event.AdjustmentEvent;
import galton.event.AdjustmentListener;
import galton.event.StartStopEvent;
import galton.event.StartStopListener;
import java.net.URL;
import java.net.MalformedURLException;
import java.awt.Color;
import java.awt.BorderLayout;
import galton.common.*;
import galton.analysis.*;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Label;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Button;
import java.awt.Insets;

/**
 * QA interface.
 */
public class Analysis extends Applet
		implements ActionListener, AdjustmentListener, StartStopListener, Runnable {
    //**************************** controls **************************//
    Frame frame;
    Options opts;           //
    Results results;        // what the machine spits out
    ControlPanel control;   // a few buttons and scrollbars to work with machine
    Machine machine;        // a thread that does most of the work
    DisplayArea display;	// the place with big red bars, was Panel panel3;
    URL helpURL;            // the URL of the help page
    Thread appletThread;    // a thread whose target is applet
    int DEBUG = 3;

    protected galton.analysis.Language lang;

    /**
     * Called when the applet is shown or becomes active.
     */
    public void start(){
		if(DEBUG>=2) System.out.println(""+machine.getSum()+": An.start()");
        if(appletThread==null){
            appletThread=new Thread(this,"Analysis");
            appletThread.start();
        	if(DEBUG>=2) System.out.println(appletThread.getName()+" started");
        } else System.out.println("An.start(): appletThread="+appletThread);
    }

    /**
     * Called when applet is hidden.
     */
    public void stop(){
		if(DEBUG>=2) System.out.println(""+machine.getSum()+": An.stop()");
		appletThread=null;
    }

	public void destroy(){
		frame.dispose();	
	}
	
    /**
     * The run method regurarly refreshes displayed components when machine is running.
     */
	public void run(){
		if(DEBUG>=2)System.out.println(""+machine.getSum()+": An.run()/begin");
        Thread thread = Thread.currentThread();
/*
		control.flipperSS.dispatchEvent(new ActionEvent(control.flipperSS,
							ActionEvent.ACTION_PERFORMED,
							lang.getName(Language.START)));
*/
        while( machine.isAlive() && thread==appletThread ){
	    	display.repaint();
	    	control.display();
        	machine.getRepaintPermit(200);			
      	}
		if(DEBUG>=2)System.out.println(""+machine.getSum()+": An.run()/end");
    }

    /**
     * For implementing ActionListener.
     * @see java.awt.event.ActionListener
     */
    public void actionPerformed(ActionEvent evt){
		String command=evt.getActionCommand();
		if(DEBUG>=2) System.out.println(""+machine.getSum()+": An.actionPerf:"+command);
		if(command.equals(lang.getName(Language.OK))){
	    	display.setDist(opts.getDist());
	    	display.repaint();
    	    return;
		}
        if(command.equals(lang.getName(Language.HELP))){
            int i=0;
            if(evt.getSource().equals(opts.helpButton))i=2;
	    //   if(evt.getSource().equals(results.getComponent(2)))i=3;
            if(evt.getSource().equals(control.getComponent(16)))i=1;
            try{
		  		URL help=new URL(helpURL,"#s1"+i);
				if(DEBUG>=1) System.out.println("Trying to load \""+help+"\"...");
				getAppletContext().showDocument(help,lang.getName(Language.HELP));
            }catch(MalformedURLException e){
				System.out.println(""+e+" at Analysis");
            }
            return;
        }
		if(command.equals(lang.getName(Language.OPTION))){
		    //    opts.setVisible(true);
		    opts.update();
		    opts.show();
	            return;
	        }
	        if(command.equals(lang.getName(Language.RESULT))){
	            System.gc();
			    int size=opts.getDist().getSize();
		            DistributionVector alldists=new DistributionVector(0);
		            for(int i=0; i<size; i++)
				if((opts.getDist().distributionAt(i)!=null)&&opts.getDist().distributionAt(i).isVisible())
		                    alldists.addDistribution(opts.getDist().distributionAt(i));
			    Distribution[] test=display.getTest();
			    for(int i=0; i<test.length; i++){
				if(DEBUG>=3) System.out.println(""+test[i].toString());
				if(test[i].isVisible())
				    alldists.addDistribution( test[i] );
			}
	    	results.display(machine,alldists);
            return;
        }
        if(command.equals(lang.getName(Language.EBAR))){
            display.toggleEBars();
            display.repaint();
            return;
        }
/*		
        if(command.equals(lang.getName(Language.START))){
            boolean ready=true;
            if(!control.fieldn.parse(false))return;
            if(!control.fieldN.parse(false))return;
            //disable components
            control.fieldn.setEnabled(false);
            control.fieldN.setEnabled(false);
            control.pBar.setEnabled(false);
            control.serButton.setEnabled(false);
            control.sumButton.setEnabled(false);
            display.repaint();
            machine.setRunning(true);
            control.getComponent(14).setEnabled(false); //options
            control.getComponent(15).setEnabled(false); //results
            return;
        }
        if(command.equals(lang.getName(Language.STOP))){
            machine.setRunning(false);
            return;
        }
        if(evt.getSource()==control.fieldn.field){
            display.columnWidthChanged=true;
            machine.setn(control.fieldn.getValue());    //automatically stops machine
        }
        if(command.equals(control.sumButton.getLabel())){
            machine.setn(control.fieldn.getValue());    //automatically stops machine
	    display.repaint();
	    return;
        }
        if(command.equals(control.serButton.getLabel())){
            machine.clearCounter();
            return;
        }
        if(command.equals(control.fieldN.COMMAND)){
            machine.setN(control.fieldN.getValue());
            return;
        }
*/
        if(command.equals(lang.getName(Language.TEST))){
            display.toggleTest();
            display.repaint();
            return;
        }
	}

    /**
     * For implementing AdjustmentListener.
     */
    public void nChanged(AdjustmentEvent e){
		System.out.println("An.nChanged("+e+")");
		if(display!=null)
			display.repaint();	
	}
    public void NChanged(AdjustmentEvent e){}
	public void clearCounter(AdjustmentEvent e){}
	/**
	 * Updates the display test distributions.
	 */
    public void pChanged(AdjustmentEvent e){
		System.out.println("An.pChanged("+e+")");
		if(display!=null){
			display.updateTest();
			Distribution[] test=display.getTest();
			if( test[0].isVisible() || test[1].isVisible() || test[2].isVisible() ) display.repaint();		
		}		
	}
	
    //******************** constructors ******************************//
	public Analysis(){
		System.out.println("Analysis.<init>");
		machine=new Machine();	//not running   
		machine.addStartStopListener(this);
	}


    /**
     * Initialises all of subcomponents, incl. lang, option, results, control.
     * @override init() in java.applet.Applet
     */
	public void init(){
		System.out.println("Analysis.init()");
	
		lang = new galton.analysis.Language(this);
		machine.NMAX=lang.getValue(Language.NMAX);

		setBackground(new Color(16096118));
		setLayout(new BorderLayout());

		frame = new FrameImproved("Analysis");
		frame.setLayout(new GridBagLayout());
		frame.setBackground(this.getBackground());	
		
		
		try{
			helpURL=new URL(getDocumentBase(),lang.getName(Language.HELPURL));
		}catch(MalformedURLException e){ System.err.println(""+e); }
		
		control=new ControlPanel(lang,machine,this);
//!		control.dispatch();
		
		GridBagConstraints c = new GridBagConstraints ();
		
		 c.fill = GridBagConstraints.BOTH;
		 c.weightx = 0.0;
		 c.weighty = 1.0;
		 c.anchor = GridBagConstraints.SOUTH;    
		 c.gridwidth = GridBagConstraints.RELATIVE;
		
	  	 c.gridheight =2;
		frame.add(control,c);
			
		display=new DisplayArea(machine,lang);
		display.setBackground( getBackground() );//new Color(16096118).brighter() ); //F59B76    
		
		 c.gridwidth = GridBagConstraints.REMAINDER;     
		 c.gridheight =1;
		 c.weightx = 1.0;
		frame.add(display,c);
		
		 c.weighty = 0.0;
		 c.insets = new Insets(0,10,0,0);
		Label frameStatus = new Label(Language.DEFAULT_STATUS);
		lang.setFrameStatus(frameStatus);
		frame.add(frameStatus,c);
		
		frame.setSize(frame.getPreferredSize());
		/*
		if(DEBUG>=2) for(int i=0;i<frame.getComponentCount();i++) {
		System.out.println("* "+i+". "+frame.getComponent(i)+" pfSize="+frame.getComponent(i).getPreferredSize().width+"|"+frame.getComponent(i).getPreferredSize().height);
		}
		System.out.println(""+frame+" pfSize="+frame.getPreferredSize().width+"|"+frame.getPreferredSize().height);
		*/
		
		frame.show();
		frame.pack();
		/*
		if(DEBUG>=2) for(int i=0;i<frame.getComponentCount();i++) {
		System.out.println("* "+i+". "+frame.getComponent(i)+" pfSize="+frame.getComponent(i).getPreferredSize().width+"|"+frame.getComponent(i).getPreferredSize().height);
		}
		System.out.println(""+frame+" pfSize="+frame.getPreferredSize().width+"|"+frame.getPreferredSize().height);
		*/
		machine.addStartStopListener(control);	
		machine.addAdjustmentListener(this);
			
		results = new Results(lang.getName(Language.RESULT),lang);
		
		opts=new Options(lang.getName(Language.OPTION),this,lang);
		
		display.setDist(opts.getDist());
		
		add(new ActionButton(lang.getName(Language.TOOPEN)){
			public void actionPerformed(ActionEvent e){
				frame.show();
			}
		},"Center"); //a bug in MSIE
		doLayout();
	};

    /**
     * Returns a description of all parameters.    
     * @return String[][] as [param. number,name=0/type=1/description=2]
     */
    public String[][] getParameterInfo(){
	    String[][] info = {
	        {"name"		, "type"		, "description"},
            {"INFO"     , "String array", "<x>,<k>,sx,sm,max"},
            {"CONTROL"  , "String array", "n,N,p,serie,sum,START,STOP,+,test,Distributions,Results"},
            {"VALUE"    , "String array", "10,1,500,500,1,10000000,50,20"},
            {"DCOLORS"  , "String array", "black,blue,cyan,darkGray,gray,green,lightGray,magenta,orange,pink,redy,white,yellow"},
            {"DNAMES"   , "String array", "binomial,normal,poisson"},
            {"DLABELS"  , "String array", "color,new,delete,ok,show?,distribution"},
            {"HELP"     , "String array", "Help,help.html"},
            {"MESSAGE"  , "String array", "p...,p...,serie[0..N]...,sum...,start/stop,N...,n...,<x>,<k>,sx,sm,maximum,error,test,distributions,results,help,version"}
	    };
	    return info;
    }

    /**
     * Returns general info about applet as a String.
     * @return Long String
     */
    public String getAppletInfo(){
        return "Analysis v."+lang.getVersionString()+"\nby Zbigniew Szmek\nzj186535@zodiac.mimuw.edu.pl";
    }
 
    /**
     * Could be used to run as a standalone application.
	 */
	 
	public static void main(String args[]){
		java.awt.Frame f=new Frame("Analysis");
		f.add(new Analysis());
		f.setSize(f.getPreferredSize());
		f.pack();
		f.show();
	}
	
	
	//************************ galton.event.Event stuff **********************************//
	public void machineStopped(StartStopEvent e){
		System.out.println("An.machineStopped("+e+")");
		stop();
		control.display(); //!
		display.repaint(); //!
	};
	public void machineStarted(StartStopEvent e){
		System.out.println("An.machineStarted("+e+")");	
		start();
	};
	
	protected abstract class ActionButton extends Button implements ActionListener{
		ActionButton(String label){
			super(label);
			addActionListener(this);
		}	
	}
}