/* 
 * @(#)ControlPanel.java
 */

package galton.analysis;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.*;
import java.text.*;
import java.applet.AppletContext;
import galton.analysis.*;
import galton.common.*;
import galton.event.*;
import galton.event.AdjustmentEvent;

/**
 * A Panel with buttons and labels and textfields in a column.
 *
 */
public class ControlPanel extends Panel
    implements ActionListener, StartStopListener {

    private static final int DEBUG = 1;

    protected Panel panel1, panel2;
    protected Label pLabel;
    protected Scrollbar pBar;
    protected Button serieButton, sumButton;
	protected Label serieLabel,sumLabel;
    protected Label[] infoLabel;
    protected Color color;
    protected Machine machine;
    protected NumberField fieldn, fieldN;
    protected StartStopFlipper flipperSS;

    protected Language lang;
	protected NumberFormat nf;
	
    private boolean first = true;
     
    //**************** constructor ********************//
    public ControlPanel (Language lang, Machine machine_, ActionListener actionListener){
		System.out.println("CP.<init>/begin");
		this.machine = machine_;
		this.lang = lang;

		nf = NumberFormat.getInstance ();
		nf.setGroupingUsed (false);
		nf.setMaximumFractionDigits (5);
		
		setLayout (new GridBagLayout ());
		GridBagConstraints c = new GridBagConstraints ();
		setFont(lang.font_normal);

	 	 c.fill = GridBagConstraints.BOTH;
	 	 c.weightx = 1.0;
	 	 c.gridwidth = GridBagConstraints.REMAINDER;
	 	 c.insets = new Insets (1, 2, 1, 2); //responsible for button padding
      
       	 c.weighty = 1.0;
		add(new Component(){
          //the extension will go here
		},c);

	
         c.weighty = 0.0;
        //****************************************************************************[   p     ]
        pBar = new Scrollbar (0, lang.getValue(Language.pEXP), 5, 0, 100 + 5);
        pLabel = new PLabel(pBar);

        pBar.addAdjustmentListener ((java.awt.event.AdjustmentListener)pLabel);
		final int id= java.awt.event.AdjustmentEvent.ADJUSTMENT_VALUE_CHANGED;
		pBar.dispatchEvent(new java.awt.event.AdjustmentEvent(pBar,id,id,pBar.getValue()));
		
		add(pLabel,c);
		add(pBar,c);

        //****************************************************************************[  serie  ]
        serieLabel = new Label(); //enforce length
        serieButton = new ActionButton(){
            public void actionPerformed(ActionEvent e){
                machine.clearCounter();
            }
        };
		machine.addAdjustmentListener(new AdjustmentAdapter(){
			public void clearCounter(galton.event.AdjustmentEvent e){ 
				System.out.println("CP.AdjustmentAdapter{}.clearCounter("+e.getTypeLabel()+")");
				display(); /* a little overhead, but who cares */ 
			}
		});
		
        serieButton.setFont (lang.font_smaller);
		serieButton.setForeground(Color.blue);		
		add(new PanelTwo(serieLabel,serieButton),c);

        //****************************************************************************[   sum    ]
        //	sumButton = new Button ();
        sumLabel = new Label();
        
        sumButton = new ActionButton(){
            public void actionPerformed(ActionEvent e){
                machine.setn();
            }
        };
        sumButton.setFont (lang.font_smaller);
		sumButton.setForeground(Color.blue);
		add(new PanelTwo(sumLabel,sumButton),c);

        //****************************************************************************[   SS    ]
		flipperSS = new StartStopFlipper (lang.getName(Language.START),lang.getName(Language.STOP));
		flipperSS.setForeground (Color.red);
		machine.addStartStopListener(flipperSS);
		add (flipperSS,c);

        //****************************************************************************[   N & n   ]
		c.insets = new Insets (1, 2, 0, 3);
		fieldn = new SmartNumberField (lang.getName(Language.nSTRING) + "=", 
						  lang.getValue(Language.nMIN),lang.getValue(Language.nMAX),lang){
			protected void valueChanged(int value){ machine.setn(value); }

		};
		fieldN = new SmartNumberField (lang.getName(Language.NSTRING) + "=", 
						  lang.getValue(Language.NMIN),lang.getValue(Language.NMAX),lang){
			protected void valueChanged(int value){ machine.setN(value); }
		};
		/*
		fieldn.addActionListener (this);
		fieldn.addActionListener (l1);
		fieldN.addActionListener (this);
		fieldN.addActionListener (l1);
		*/
		fieldn.setValue (lang.getValue(Language.nEXP));	// fieldn.parse();
		fieldN.setValue (lang.getValue(Language.NEXP));	// fieldn.parse();
		
		System.out.println("CP.<init>: manually setting n and N");
		machine.setn(lang.getValue(Language.nEXP));
		machine.setN(lang.getValue(Language.NEXP));
	
		add (fieldN,c);
		add (fieldn,c);
	
        //****************************************************************************[   info  ]
		c.insets = new Insets (0, 2, 0, 0);
		infoLabel = new Label[5];
		for (int i = 0; i < 5; i++) {
		    infoLabel[i] = new Label (makeWide (lang.getName(Language.INFO+i) + " : ", "-", 12), Label.LEFT);
		    infoLabel[i].setFont (lang.font_info);
		    add (infoLabel[i],c);
		}
        //****************************************************************************[  + & test ]
		c.insets = new Insets (0, 1, 0, 0);
//		c.gridwidth = GridBagConstraints.REMAINDER;
        add(new Button(lang.getName(Language.EBAR)),c);
        add(new Button(lang.getName(Language.TEST)),c);	
        //****************************************************************************[  window openers ]
		c.insets = new Insets (2, 1, 0, 0);
        add(new Button(lang.getName(Language.OPTION)),c);	
		c.insets = new Insets (0, 1, 0, 0);
        add(new Button(lang.getName(Language.RESULT)),c);	
        add(new Button(lang.getName(Language.HELP)),c);	
		
		for(int i=0;i<getComponentCount();i++){
			if(getComponent(i) instanceof Button) ((Button)getComponent(i)).addActionListener(actionListener);
		}
	
        //****************************************************************************[  helper initialization ]
      	CPMouseListener mouseListener = new CPMouseListener();
		this.addMouseListener (mouseListener);
		for (int i = 0; i < getComponentCount (); i++)
		  	   getComponent (i).addMouseListener (mouseListener);
		serieLabel.addMouseListener(mouseListener);
		serieButton.addMouseListener(mouseListener);
		sumLabel.addMouseListener(mouseListener);
		sumButton.addMouseListener(mouseListener);		
					   
		// enumerateComponents();
		System.out.println("CP.<init> calling display()");
		display();	
		System.out.println("CP.<init>/end");
	}
	
/*	
	public void dispatch () {
		if(DEBUG>=2) System.out.println (""+machine.getSum()+": ControlPanel.dispatch");
		fieldn.parse (true);
		fieldN.parse (true);
		pBar.dispatchEvent (new AdjustmentEvent (pBar, 
							 AdjustmentEvent.ADJUSTMENT_VALUE_CHANGED, 
							 AdjustmentEvent.TRACK, pBar.getValue ()));
	}
*/	
	
	
	//*************** paint methods *******************//
	public void display() {
		if(serieLabel == null || sumLabel==null || infoLabel==null) return;
		nf.setMinimumFractionDigits (0);
		serieLabel.setText ( lang.getName(Language.SERIE) + " : " + (machine.getSerie()==0 ? "                 0" : nf.format (machine.getSerie ())));
		sumLabel.setText (lang.getName(Language.SUM) + " : " + (machine.getSum()==0 ? "                 0" : nf.format (machine.getSum ())));
		//        sumLabel.setLabel(makeWide(SUM+" : ",""+machine.getSum(),18));
		//        boolean isHuge=machine.getSum()>=(10*NRANGE[2]);
		//        nf.setMaximumIntegerDigits( (int)(Math.log(NRANGE[2])/Math.log(10)));
		//        serieLabel.setText    (SERIE  +" : "  +nf.format(machine.getSerie()) );
		//        sumLabel.setText      (SUM    +" : "  +nf.format(machine.getSum())   );
		/*
		if (!machine.isRunning ()) {
		    flipperSS.a ();
		    for (int i = 0; i < this.getComponentCount (); i++)
			this.getComponent (i).setEnabled (true);
		}
		*/
		machine.makeInfo ();
		nf.setMinimumFractionDigits (3);
		for (int i = 0; i < 4; i++)
		    infoLabel[i].setText (makeWide (lang.getName(Language.INFO+i) + " : ", 
						    nf.format (machine.getInfo (i)), 12));
		nf.setMinimumFractionDigits (0);
		infoLabel[4].setText(makeWide (lang.getName(Language.INFO5) + " : ",
	                           nf.format (machine.getInfo (4)), 12));
	} 
	
	//*************************** methods ***************************//
	protected String trimDouble (double val, int places) {
		String str = "" + val;
		int i = str.indexOf ('.');
		i++;
		boolean useSubstring = i + places <= str.length ();
		if (i == -1)
		    useSubstring = false;
		String res = useSubstring ? str.substring (0, i + places) : str;
		if (useSubstring)
		    return str.substring (0, i + places);
		else
		    return str;
	}
	protected String makeWide (String beg, String end, int length) {
		int i;
		String res = new String (beg);
		for (i = length - beg.length () - end.length (); i > 0; i--)
		    res += " ";
		res += end;
		return res;
	    }
	
	protected void enumerateComponents(){
	     for (int i = 0; i < getComponentCount (); i++){
	           System.out.println(""+getComponent(i)+" pfSize="+getComponent(i).getPreferredSize());
	      }
	}
	
	
	
	//************************ galton.event.Event stuff **********************************//
	public void machineStopped(StartStopEvent e){
		System.out.println("CP.machineStopped("+e+")");
		for(int i=0;i<getComponentCount();i++) getComponent(i).setEnabled(true);
	};
	public void machineStarted(StartStopEvent e){
		System.out.println("CP.machineStarted("+e+")");
		pBar.setEnabled(false);
		for(int i=0;i<getComponentCount();i++)
			if(getComponent(i) instanceof Button && ((Button)getComponent(i)).getLabel()!=lang.getName(Language.HELP)
												 && !(getComponent(i) instanceof StartStopFlipper)) 
				getComponent(i).setEnabled(false);
		fieldn.setEnabled(false);
		fieldN.setEnabled(false);
		serieButton.setEnabled(false);
		sumButton.setEnabled(false);
	};
	    
	
	//************************ java.awt.AWTEvent handlers *******************************//
	
	public void actionPerformed (ActionEvent evt) {
		if(DEBUG>=2) System.out.println (""+machine.getSum()+": CP.actionPerf:" + evt.getActionCommand ());
		if(DEBUG>=2) System.out.println (""+machine.getSum()+": fieldN:" + evt.getSource ().equals (fieldN.field));
	
		if (evt.getSource().equals(fieldN.field)) {
		    machine.setN (fieldN.getValue ());
		    serieButton.setLabel (lang.getName(Language.SERIE)+" : "+NumberFormat.getInstance().format (machine.getSerie()));
		    return;
		} 
    }

    protected class CPMouseListener extends MouseAdapter{
		public void mouseExited (MouseEvent e) {
//			System.out.println("CP.CPMouseListener.mouseExited("+e.getComponent()+"/"+getComponentCount()+")");			
			lang.showFrameStatus (Language.DEFAULT_STATUS);
		}
		public void mouseEntered (MouseEvent e) {
//			System.out.print("CP.CPMouseListener.mouseEntered("+e.getComponent()+"/"+getComponentCount()+")");
			int skip=1; // 1 because of "A" component			
			for (int i = 1; i < getComponentCount (); i++)
				if (getComponent(i).equals (e.getComponent ())) {
//					System.out.println("->"+(i-skip));
					lang.showFrameStatus (lang.getMessage(i-skip)); 
					return;
				} else { 
					if (e.getComponent() instanceof PanelTwo) skip +=2;
				}
//			System.out.println();
			if(serieLabel==e.getComponent()) { lang.showFrameStatus(lang.getMessage(2)); return; }
			if(serieButton==e.getComponent()){ lang.showFrameStatus(lang.getMessage(17)); return; }
			if(sumLabel==e.getComponent())   { lang.showFrameStatus(lang.getMessage(3)); return; }
			if(sumButton==e.getComponent())  { lang.showFrameStatus(lang.getMessage(18)); return; }
			
			lang.showFrameStatus(lang.getMessage(Language.MESSAGES-1));
		}
	}

    protected class PLabel extends Label implements java.awt.event.AdjustmentListener{
        protected NumberFormat nf;
        protected Scrollbar scrollbar;
        public PLabel(Scrollbar scrollbar){
			System.out.println("PLabel.<init>");
            this.scrollbar = scrollbar;
            nf = NumberFormat.getInstance ();
            nf.setMaximumFractionDigits (2);
            nf.setMinimumFractionDigits (2);
        }

        public void adjustmentValueChanged (java.awt.event.AdjustmentEvent evt) {
			String val = nf.format ((double) scrollbar.getValue () / 100.0);
			String text = lang.getName(Language.pSTRING).concat(" = ").concat (val);
			if ( getText()!=text ) setText (text);
			machine.setp(scrollbar.getValue()/100.0);
        } 
    }

    protected abstract class ActionButton extends Button implements ActionListener{
         public ActionButton(){
             super("X");
             addActionListener(this);
         }
    }

	protected class StartStopFlipper extends ButtonFlipper implements StartStopListener{
		public StartStopFlipper(String start,String stop){ super(start,stop); }
		
		public void machineStarted(StartStopEvent e){ b(); }
		public void machineStopped(StartStopEvent e){ a(); }
		
		public void actionPerformed(ActionEvent e){ 
//			System.out.println("StartStopFlipper.actionPerformed("+e.getActionCommand()+")");
			super.actionPerformed(e);
        	if(e.getActionCommand().equals(A)){
            	b(); machine.start();
        	}else{
            	a(); machine.stop();
			}
		}
	}//8473769
	protected class PanelTwo extends Panel{
		public PanelTwo(Component left,Component right){
			super(	new GridBagLayout() );
			GridBagConstraints c = new GridBagConstraints();
			 c.weightx=1.0;
			 c.gridwidth=GridBagConstraints.RELATIVE;
			 c.fill = GridBagConstraints.BOTH;
			add(left,c);
			 c.weightx=0.0;
			 c.gridwidth=GridBagConstraints.REMAINDER;
			add(right,c);
		}
		public void setEnabled(boolean enabled){
			getComponent(0).setEnabled(enabled);
			getComponent(1).setEnabled(enabled);
		}
	}
	protected abstract class SmartNumberField extends NumberField implements AdjustmentListener{
		public SmartNumberField (String text, int min, int max,Parse parse) {super(text,min,max,parse); }
		
		public void pChanged(AdjustmentEvent e){}      // at least one of these methods should be overridden!
		public void nChanged(AdjustmentEvent e){}
		public void NChanged(AdjustmentEvent e){}
		public void clearCounter(AdjustmentEvent e){}
		protected void errorDetected(Object param){
			System.out.println("CP$SmartNumberField.errorDetected("+param+")");

			String message_;
			if (param instanceof Integer){
				int newValue = ((Integer)param).intValue();
				message_ = lang.getMessage(20)+" ( "+(newValue<min?newValue+" < "+min : newValue+" > "+max)+" ) " ;
			} else 
				message_ = lang.getMessage(19)+"("+param+")";
			lang.showFrameStatus(message_);
			getToolkit().beep();
			
			Container f = getParent();
			while(f!=null && !(f instanceof Frame)) f=f.getParent();
			if(box!=null) box.dispose();
			box = new MessageBox(f!=null ? (Frame)f : null,"",lang.getName(Language.OK),message_);
			System.out.println("MessageBox <init>ed");
			System.out.println(""+f);

			new Thread(
			new Runnable(){
				public void run(){ 	box.show();	} /* will return after box is closed */
			},"MessageBoxOpener").start();			

			System.out.println("MessageBox (newThread)shown");
			box.setSize(box.getPreferredSize());
			box.pack();
			box.setResizable(false);
			System.out.println(""+box+" modal="+box.isModal());
			message_ = null;
		}
		protected MessageBox box;
	}
}
