/*
 * @(#)DisplayArea.java
 */
package galton.analysis;

import java.awt.Panel;
import java.awt.Graphics;
import java.awt.Cursor;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Dimension;
import java.awt.Color;
import java.awt.FontMetrics;
import java.text.NumberFormat;
import java.applet.AppletContext;
import java.awt.event.*;
import galton.analysis.Distribution;
import galton.analysis.DistributionVector;
import galton.analysis.Machine;
import galton.analysis.Language;

/**
 * A panel displaying machine in visual form. Capable also of displaying continous and 
 * discreet distributions and even error bars.
 */
public class DisplayArea extends Panel implements MouseMotionListener{

    public DisplayArea(Machine machine,Language lang){
	  if(DEBUG>=2) System.out.println("DA.<init>/begin"); 
	  this.lang = lang;
	
	  this.machine=machine;
	  eBarsVisible=false;
      columnWidth=-1;
      setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
      test=new Distribution[3];
      for(int i=0;i<3;i++) test[i]=new Distribution(lang);
      columnWidthChanged=true;
      addMouseMotionListener(this);
	  if(DEBUG>=3) System.out.println("DA.size(): "+getSize());  
	  if(DEBUG>=2) System.out.println("DA.<init>/end");	
    }
	
    private void drawArrow(Graphics g,int x,int y,int r,int orientation,int arl,Color color) {
        g.setColor(color);
        if(orientation==HORIZONTAL){
            g.drawLine(x-arl,y,x+r-arl,y);
            g.drawLine(x-arl,y+1,x+r-arl,y+1);
            int[] xPoints={x+r,x+r-3*arl,x+r-3*arl,x+r};
            int[] yPoints={y,y-arl,y+arl,y};  //must be same length as xPoints
            g.drawPolygon(xPoints,yPoints,xPoints.length);
        }else{
            g.drawLine(x+1,y+arl,x+1,y-r+arl);
            g.drawLine(x,y+arl,x,y-r+arl);
            int[] xPoints={x,x-arl,x+arl,x};
            int[] yPoints={y-r,y-r+3*arl,y-r+3*arl,y-r};// must be same length as xPoints
            g.drawPolygon(xPoints,yPoints,xPoints.length);
        }
    }
    private int getNextStep(int lastStep){
        double step=(double)lastStep+1.0;
        boolean ok=false;
        while(!ok){
            step++;
            ok=false;
            if(Math.abs(Math.floor(Math.log(step)/Math.log(10.0))-Math.log(step)/Math.log(10.0))<0.0000000000000005)ok=true;
            if(Math.abs(Math.floor(Math.log(step)/Math.log(10.0))-Math.log(step)/Math.log(10.0)+1)<0.0000000000000005)ok=true;
            if(Math.abs(Math.floor(Math.log(2*step)/Math.log(10.0))-Math.log(2*step)/Math.log(10.0))<0.0000000000000005)ok=true;
            if(Math.abs(Math.floor(Math.log(2*step)/Math.log(10.0))-Math.log(2*step)/Math.log(10.0)+1)<0.0000000000000005)ok=true;
            if(Math.abs(Math.floor(Math.log(5*step)/Math.log(10.0))-Math.log(5*step)/Math.log(10.0))<0.0000000000000005)ok=true;
            if(Math.abs(Math.floor(Math.log(5*step)/Math.log(10.0))-Math.log(5*step)/Math.log(10.0)+1)<0.0000000000000005)ok=true;
        }
        return (int)step;
    }
    private void drawAxisY(Graphics g){
		NumberFormat pf=NumberFormat.getPercentInstance();
		pf.setMaximumFractionDigits(1);
        int SPACES=2;
        g.clearRect(0,0,Origin.x+ARROWSIZE,Origin.y);  //
        drawArrow(g,Origin.x,Origin.y,Origin.y,VERTICAL,ARROWSIZE,ARROW_COLOR); //
        g.setColor(YMARKER_COLOR); //
        int xPoint=Origin.x-getFontMetrics(getFont()).stringWidth(ySTRING)-ARROWSIZE; //
        g.drawString(ySTRING,
        		xPoint>0?xPoint:(int)0, 3*ARROWSIZE); //
        if(top<=1){
	        g.setColor(YMARKER_COLOR);
            g.drawString(""+0,Origin.x-g.getFontMetrics().stringWidth(""+0),Origin.y);
			g.drawString(""+1,Origin.x-g.getFontMetrics().stringWidth(""+1),
							  Origin.y-Port.height+getFontMetrics(getFont()).getAscent());
        }else{
            int step=1;
            while(top*getFontMetrics(getFont()).getAscent()*SPACES>step*Port.height)
                step=getNextStep(step);
            g.setColor(YMARKER_COLOR);
			int ascent=getFontMetrics(getFont()).getAscent();
			double sum = machine.getSum();
            for(int i=0;i<(int)(top/step);i++){
                int h=i*step*Port.height/top;
                xPoint=Origin.x-g.getFontMetrics().stringWidth(""+step*i);
        		g.setColor(YMARKER_COLOR); //
                g.drawString(""+(int)(step*i),xPoint>0?xPoint:0,Origin.y-h);				
				if(i>0){
					g.setColor(YMARKER_COLOR.brighter());				
					g.drawString(pf.format((double)step*(double)i/sum),xPoint>0?xPoint:0,Origin.y-h+ascent);
                	g.drawLine(Origin.x-ARROWSIZE,Origin.y-h,Origin.x+ARROWSIZE-1,Origin.y-h);
				}
	        }
        }
    }
    private void drawAxisX(Graphics g){
		double SPACES=1.5;
		FontMetrics metrics=g.getFontMetrics();
		NumberFormat nf=NumberFormat.getInstance();
		int markerWidth=metrics.stringWidth(nf.format(machine.ktox(0)));
		Color sysColor=g.getColor();
		g.clearRect(0,Origin.y,Bounds.width,Bounds.height-Origin.y);
		int _every=(int)Math.ceil((double)markerWidth*SPACES/(double)columnWidth);
		int ypoint=Origin.y+metrics.getMaxAscent();
		for(int k=getFirstMarker(machine,_every);k<=machine.n;k+=_every){
			int xcenter=Origin.x+ARROWSIZE+ (int)( ((double)k+0.5)*(double)columnWidth );
			g.drawLine(xcenter,Origin.y,xcenter,Origin.y+ARROWSIZE/2);
			g.setColor(XMARKER_COLOR);
			g.drawString(nf.format(machine.ktox(k)),
				xcenter-metrics.stringWidth(nf.format(machine.ktox(k)))/2, ypoint+ARROWSIZE);
			g.setColor(KMARKER_COLOR);
			g.drawString(nf.format(k),
				xcenter-metrics.stringWidth(nf.format(k))/2, ypoint+metrics.getMaxAscent()+ARROWSIZE);
			g.setColor(sysColor);
		}
		drawArrow(g,Origin.x,Origin.y,Bounds.width-Origin.x,HORIZONTAL,ARROWSIZE,ARROW_COLOR);
        g.setColor(XMARKER_COLOR);
        g.drawString(lang.getName(Language.xSTRING),
                Bounds.width-ARROWSIZE-metrics.stringWidth(lang.getName(Language.xSTRING)),
                Origin.y+metrics.getMaxAscent()+ARROWSIZE);
        g.setColor(KMARKER_COLOR);
        g.drawString(lang.getName(Language.kSTRING),
                Bounds.width-ARROWSIZE-metrics.stringWidth(lang.getName(Language.kSTRING)),
                Origin.y+2*metrics.getMaxAscent()+ARROWSIZE);
    }
    private int getFirstMarker(Machine m,int _every){
        int a=(int)(-m.ktox(0)-Math.floor(-m.ktox(0)/(double)_every)*(double)_every);
        return a<0?0:a;
    }
    public void toggleEBars(){
	eBarsVisible=!eBarsVisible;
    }
    public void paint(Graphics g){
	  if(DEBUG>=2) System.out.println("DA.paint()");
      super.paint(g);
	  Bounds=this.getBounds();
	  Origin=new Point(g.getFontMetrics().stringWidth(""+machine.NMAX*5),
    	    Bounds.height-getFontMetrics(getFont()).getMaxAscent()*2-ARROWSIZE);
	  Port=new Rectangle(Origin.x+ARROWSIZE,3*ARROWSIZE,Bounds.width-Origin.x-4*ARROWSIZE,Origin.y-3*ARROWSIZE).getSize();
//        new Dimension(Bounds.width-Origin.x-(4+1)*ARROWSIZE,Origin.y-3*ARROWSIZE);
      columnWidthChanged=true;
	  update(g);
	}
	public void update(Graphics g){
		if(DEBUG>=3) System.out.println("DA.update("+g+")");
		if((Bounds==null) || (Origin==null)) { paint(g); return; }

		Graphics port=g.create(Origin.x+ARROWSIZE,3*ARROWSIZE,Bounds.width-Origin.x-4*ARROWSIZE,Origin.y-3*ARROWSIZE);
		port.setColor(Color.red);
		int _columnWidth=Port.width/(machine.n+1);
		if(DEBUG>=3) System.out.println("DA.update():columnWidthChanged="+columnWidthChanged+"machine isAlive:"+machine.isAlive());
		if(columnWidthChanged || (_columnWidth!=columnWidth)){
			columnWidth=_columnWidth;
			columnWidthChanged=false;
			drawAxisX(g);
			updateTest();
		}
		g.drawString("|<-"+columnWidth+"->|",0,Origin.y+2*ARROWSIZE);
		if(machine.isAlive() && machine.max<top){
			int[] Modif=machine.getModified();
			for (int i=0;i<Modif.length;i++)
			drawColumn(port,Modif[i]);
		}else{
			port.clearRect(0,0,g.getClipBounds().width,g.getClipBounds().height);
			top=machine.getTop();
			long sum=machine.getSum();
			if(sum!=0)
				percentTop=(double)top/(double)sum;
			else
				percentTop=1.0;
			for(int i=0;i<=machine.n;i++) drawColumn(port,i);
			drawAxisY(g);
		}
		if(!machine.isAlive()){
			for(int i=0;i<3;i++)
				if(test[i].isVisible()) 
					drawDistribution(port,test[i]);
				if(dist!=null)
					for(int i=0;i<dist.getSize();i++)
						if((dist.distributionAt(i)!=null) && dist.distributionAt(i).isVisible())
							drawDistribution(port,dist.distributionAt(i));
		}
	}
	
    private void drawDistribution(Graphics port,Distribution d){
	  if(!d.isContinous()){
	    for(int k=0;k<=machine.n;k++)
		drawDColumn(port,k,
			    d.getValue((double)k,machine.ktox(k)),
			    d.getColor(),d.isSpecial());
      }else{
	    double oldval=-1;
	    double dh=machine.n*1.0/(double)Port.width;
	    for(double k=-0.5;k<=(double)machine.n+0.5;k+=dh*(double)C_PIXELSKIP)
		oldval=drawCColumn(port,k,
				   d.getValue(k,machine.ktox(k)),oldval,
				   d.getColor(),d.isSpecial());
	  }
    }
    private synchronized boolean drawColumn(Graphics port,int i){
	    int columnHeight=machine.getValue(i)*Port.height/top;
//        if(DEBUG>=3) System.out.println("Port.height:"+Port.height+"columnHeight:"+columnHeight);
	    Rectangle rect=new Rectangle(	i*columnWidth,	Port.height-columnHeight,
										columnWidth,	columnHeight			);
		port.setColor(COLUMN_COLOR);
	    if(columnWidth>=4){
		    port.fill3DRect( rect.x+1,rect.y,rect.width-1,rect.height,true );
		}
		else if(columnWidth==2 || columnWidth==3)
		    port.fillRect(rect.x+1,rect.y,rect.width-1,rect.height);
		else if(columnWidth==1){
		    port.drawLine(rect.x,rect.y,rect.x,rect.y+rect.height);
			return true;
		}
	    else
		    return false;
		if(eBarsVisible && (columnHeight!=0)){
			int eBarHeight=(int)Math.rint(Math.sqrt(machine.getValue(i))*(double)Port.height/(double)top);
		    Rectangle eBar=new Rectangle( 	rect.x+1+(rect.width-2)/2,rect.y-eBarHeight,
                                            rect.x+1+(rect.width-2)/2,rect.y+eBarHeight);
			port.setColor(EBAR_COLOR);
			port.drawLine( eBar.x,eBar.y,eBar.width,eBar.height );
			if(!machine.isAlive() && columnWidth>=4)
			    port.drawArc( eBar.x-5,(eBar.y+eBar.height)/2-5,10,10,0,360);
		}
	    return true;
    }
	private synchronized double drawCColumn(Graphics port,double k,double val,double oldval,Color color, boolean special)
	{
	
		if( oldval<0){
			return val;
		}else{
		    Color sysColor=port.getColor();
            port.setColor(color);
		    int colHeight=(int)((double)Port.height*val/percentTop);
  		    int oldHeight=(int)((double)Port.height*oldval/percentTop);
  			for(int i=-C_THICKNESS/2;i<=-C_THICKNESS/2+C_THICKNESS;i++){
			    port.drawLine(	(int)((k+0.5)*(double)columnWidth-C_PIXELSKIP),	Port.height-oldHeight+i,
                        	    (int)((k+0.5)*(double)columnWidth), 			Port.height-colHeight+i);
    		}
            if(special){
                port.setColor(  Color.black);
			    port.drawLine(	(int)((k+0.5)*(double)columnWidth-C_PIXELSKIP),	Port.height-oldHeight,
                        	    (int)((k+0.5)*(double)columnWidth), 			Port.height-colHeight);
            }
		    port.setColor(sysColor);
		    return val;
		}
	}
	// drawDistributedColumn (used to display type 0 and 2 distribution (not continuous)
	private synchronized boolean drawDColumn(Graphics port,int k,double val,Color color,boolean special){
		Color sysColor=port.getColor();
		port.setColor(color);
		int columnHeight=(int)((double)Port.height*val/percentTop);
		Point dMark=new Point( (int)(((double)k+0.5)*columnWidth),Port.height-columnHeight );
		if(columnWidth>12){
			port.fillOval(dMark.x-5,dMark.y-5,10,10);
            if(special){
                port.setColor(Color.black);
                for(int i=0;i<4;i++){
                    port.drawArc(dMark.x-5,dMark.y-5,10,10,	45+90*i,60);
                    port.drawArc(dMark.x-4,dMark.y-4,8,8,   45+90*i,60);
                }
            }
		}else{
		    if(columnWidth>7){
				port.drawArc(dMark.x-columnWidth/4,dMark.y-columnWidth/4,columnWidth/2,columnWidth/2,0,360);
                if(special){
                    port.setColor(Color.black);
                    for(int i=0;i<4;i++)
                        port.drawArc(dMark.x-columnWidth/4,dMark.y-columnWidth/4,columnWidth/2,columnWidth/2,45+90*i,60);
                }
            }
            port.setColor(color);
			port.drawLine(dMark.x-columnWidth*2/5+1,dMark.y,dMark.x+columnWidth*2/5+1,dMark.y);
			port.drawLine(dMark.x,dMark.y-columnWidth/2+1,dMark.x,dMark.y+columnWidth/2+1);
            if(special){
                port.setColor(Color.black);
			    port.drawLine(dMark.x-columnWidth*2/5,dMark.y,dMark.x-columnWidth/4,dMark.y);
			    port.drawLine(dMark.x,dMark.y-6,dMark.x,dMark.y-columnWidth/4);
			    port.drawLine(dMark.x+columnWidth/4,dMark.y,dMark.x+columnWidth*2/5,dMark.y);
			    port.drawLine(dMark.x,dMark.y+columnWidth/4,dMark.x,dMark.y+6);
            }
		}
		port.setColor(sysColor);
		return true;
	}
	public synchronized void setDist(DistributionVector dist){
		this.dist=dist;		
	}
    public synchronized void updateTest(){
	for(int i=0;i<3;i++){
        test[i].generate(i,machine);
        test[i].setTitle(""+lang.tChar+(i+1));
	}
    }
    public void toggleTest(){
		if(DEBUG>=3)System.out.println("DA.toggleTest()* visible="+isVisible()+" size="+getSize());
        for(int i=0;i<3;i++)
            test[i].setVisible(!test[i].isVisible());
    }
    public Distribution[] getTest(){
	if(DEBUG>=2) System.out.println("DA.getTest");
        return test;
    }


    public Dimension getPreferredSize(){ return new Dimension(600,450); }

    private boolean 			eBarsVisible;	//error bars visible
    public Machine 				machine;
    protected DistributionVector	dist;
    protected Distribution[]    	test;
    protected Point 			Origin;
    protected Rectangle 		Bounds;
    protected Dimension 		Port;
    protected static final int 	ARROWSIZE=10;
    protected static final Color YMARKER_COLOR=	Color.blue;
    protected static final Color XMARKER_COLOR=	Color.blue;
    protected static final Color KMARKER_COLOR=	Color.red;
    protected static final Color ARROW_COLOR=  	Color.magenta;
    protected static final Color EBAR_COLOR=	Color.black;
    protected static final Color COLUMN_COLOR= 	Color.red;
    protected static final int 	C_THICKNESS=3;
    protected static final int	C_PIXELSKIP=3;
    protected final static int 	HORIZONTAL=0;
    protected final static int 	VERTICAL=1;
    protected int 				columnWidth;
    public	  boolean           columnWidthChanged;
    protected int 				top;
    protected double 			percentTop;
    protected Language lang;
    private static final int DEBUG=1;

    private static final String ySTRING="N(k)";
    //****************************** MouseMotionListener ************************//
    public void mouseDragged(MouseEvent evt){}
    public void mouseMoved(MouseEvent evt){
        if(Origin!=null && columnWidth>0){
        	evt.translatePoint(-Origin.x-ARROWSIZE,-3*ARROWSIZE);
            int k=evt.getX()/columnWidth;
            if(evt.getY()>0 && (evt.getY()<Origin.y-3*ARROWSIZE) && k>=0 && k<=machine.n)
	        	lang.showFrameStatus("k="+(evt.getX()/columnWidth)+" N(k)="+machine.getValue(evt.getX()/columnWidth));
            else
                lang.showFrameStatus("");
        }
    }
}

