/*
 * @(#)Distribution.java
 */
package galton.analysis;

import java.awt.Color;
import java.util.*;
import java.text.*;
import galton.analysis.Language;

/**
 * A class completly describing a distribution.
 */
public class Distribution{
    private Language lang;
    public static final int DEBUG = 0;
    public static String TYPE_NAMES[]={"Binomial","Gauss","Poisson"};
    public static final int GAUSS=1;
    public static final int BINOMIAL=0;
    public static final int POISSON=2;
    public static final int NORMAL=GAUSS;	//ALIAS
    public static final int BERNOULI=BINOMIAL;	//ALIAS
    protected int type;	                        //0,1,2, as above
    public String name;
    protected double p1,p2;
    public static String PARAM_NAMES[][]={
	{"n","p"},
	{"delta","<x>"},
	{"lambda",""}};
    static final Color[] COLORS = {Color.black,Color.blue,Color.cyan,Color.darkGray,Color.gray,Color.green,
				   Color.lightGray,Color.magenta,Color.orange,Color.pink,
				   Color.red,Color.white,Color.yellow};
    public boolean visible;
    public Color color;
    public boolean special;                     //'is a test distribution?'

    // CONSTRUCTORS //
    public Distribution(Language lang){
      this.lang = lang;
	this.type=0;
	this.color=Color.green;
	this.visible=true;      //was false (maybe better, but quite annoying)
        this.special=false;
    }
    public void generate(int newType,Machine m){
        setType(newType);
        this.special=true;
        switch(newType){
            case BERNOULI :{
                setParam(1,m.n);
                setParam(2,m.p);
                this.color=Color.blue;
            };break;
            case GAUSS :{
                setParam(1,Math.sqrt((double)m.n*m.p*(1.0 -m.p)));
                setParam(2,m.ktox((double)m.n*m.p));
                this.color=Color.green;
            };break;
            case POISSON :{
                setParam(1,m.n*m.p);
                this.color=Color.magenta;
            };break;
        }
    }
    // parameters
    public double getParam(int paramNum) throws IndexOutOfBoundsException {
	switch(paramNum){
	case 1 : return p1;
	case 2 : return p2;
	default : throw	new IndexOutOfBoundsException(""+paramNum);
	}//switch
    }
    public String setParam(int num, double val) throws IndexOutOfBoundsException{ // throws IllegalArgumentException{
	switch(num){
	case 1: p1=val;
	    return "p1 changed to "+val;
	case 2: p2=val;
	    return "p2 changed to "+val;
	default : throw new IndexOutOfBoundsException(""+num);
	}
    }
    public String getParamName(int num) throws IndexOutOfBoundsException{
	switch(num){
	case 1 : return PARAM_NAMES[getType()][num-1];
	case 2 : return PARAM_NAMES[getType()][num-1];
	default : throw	new IndexOutOfBoundsException(""+num);
	}//switch
    }
    public boolean useSecondParam(){
	return type!=2;
    }
    public boolean isVisible(){return visible;};
    public void setVisible(boolean visible){
	this.visible=visible;
	if(DEBUG>=2) System.out.println("Db.setVisible("+visible+")");
    };
    // type
    public int getType(){
	return type;
    }
    public boolean isContinous(){
	return getType()==GAUSS;
    }
    public boolean isSpecial(){
        return special;
    }
    public void setType(int type) throws IllegalArgumentException{
	if(type>=0 && type<=2)this.type=type;
	else throw new IllegalArgumentException("type "+type);
    }
    public String getTypeName(){
      if(DEBUG>=2) System.out.println("getTypeName()="+getType());
      if(DEBUG>=2) System.out.println("lang:"+lang);
	return lang.getName(Language.DNAMES+getType());
    }
    public String getTypeName(int num) throws IndexOutOfBoundsException {
	if(num>=0 && num<=2 )return lang.getName(Language.DNAMES+num);
	else throw new IndexOutOfBoundsException(""+num);
    }
    // values
    private static double power(int s){
	double r=1;
	for(s=s;s>1;s--)r*=(double)s;
	return r;
    }
    public String getTitle(){
	return name;
    }
    public void setTitle(String name){
	this.name=name;
    }
  
    public double getValue(double k,double x){//warning : different then x in Gauss
	double i;
	switch(getType()){
	case 0 :{
	    double bn=(double)p1;
	    double bp=(double)p2;
	    double a=0;
	    double b=Math.pow(bp,k)*Math.pow(1.0 -bp,bn-k);
	    double log_sum=0;
	    for(i=bn;i>1;i--)log_sum+=Math.log(i);
	    for(i=k;i>1;i--)log_sum-=Math.log(i);
	    for(i=bn-k;i>1;i--)log_sum-=Math.log(i);
	    a=Math.pow(Math.E,log_sum);
	    return a*b;
	}
	case 1 :{
	    double gd=(double)p1;
	    double ga=(double)p2;
	    return Math.exp(-Math.pow(x-ga,2)/2.0/Math.pow(gd,2))/gd/Math.sqrt(2.0*Math.PI);
	}
	case 2 :{
	    double pl=(double)p1;
	    double a=1;
	    for(i=0;i<k;i++) a*= pl / (k-i);
	    return a/Math.pow(Math.E,pl);
	}
	}
	return 0;
    }
    public String setColor(String color){
      this.color = COLORS[lang.getColorNumber(color)];     
	return "Color changed to "+getColorName();
    }
    public Color getColor(){
	return color;
    }
    public String getColorName(){
	for(int i=0;i<COLORS.length;i++)
	    if(getColor()==COLORS[i])return lang.getColorName(i);
	return "Green";
    }
    public String toString(){
	/*		String result="--\""+name+"\" :"+getTypeName()+" ("+visible+")\n"
			+getType()+", "+visible+"; "+getParam(1)+","+getParam(2)+", "+color;
	*/
        NumberFormat nf=NumberFormat.getInstance();
        String result=	"\""+name+"\"\n"+
	    getTypeName()+", "+getColorName()+"\n"+
	    getParamName(1)+" = "+nf.format(getParam(1));
        if(useSecondParam())
            result+="\n"+getParamName(2)+" = "+nf.format(getParam(2));
	return result.concat("\n");
    }
    public String ID(){
	  if(name==null) return null; else {
  	    int i=name.lastIndexOf(" ");
	    String num=name.substring(i>=0?i+1:1);
	    return name.substring(0,1).concat(num).toUpperCase();
      }
    }
}

