/*
 * @(#)Machine.java
 */
package galton.analysis;

import java.util.Random;
import java.lang.Thread;
import galton.event.*;
/**
 * A Thread used to generate random distributions.
 *
 */
public class Machine extends MachineBase implements Runnable{  

    private Random random;      // pseudorandom number generator
//!    private boolean running;   
    private boolean dead=false; //will be used to gracefully exit;

    protected Thread thisThread; //used to exit gracefully


    private boolean AnReadyToWait;

    public int DEBUG=2;

    /**
     * A standard constructor.
     * @param running whether should be running after construction.
     */
    public Machine(){
        random = new Random();
    }

//!
/*
    public boolean isRunning(){
      return running;
    }
*/
/*
    public synchronized void setRunning(boolean running){
	if(DEBUG>=1) System.out.println(""+getSum()+": Mach.setRun("+running+")/beg AnRd2Wait="+AnReadyToWait);
      if (running) AnReadyToWait = false;
      this.running=running;
	if( !thisThread.isAlive() && running) {
          start();
          processEvent(new StartStopEvent(getSum(),StartStopEvent.STOP));
      }
      notifyAll();
	if(DEBUG>=1) System.out.println(""+getSum()+": Mach.setRun(.)/end AnRd2Wait="+AnReadyToWait);
    }
*/  

    /**
     * A method called to measure out the time between repaintings.
     * @param time the expected time to wait
     * @param readyToWait if the Object calling has repainted after Machine has stopped running
     */
     public synchronized void getRepaintPermit(int time){
         try{
             wait(time); // else wait();
         }catch(InterruptedException e){
         }
     }

/*
    public synchronized void getRepaintPermit(int time){
      boolean running=false;
	if(DEBUG>=2) System.out.println(""+getSum()+": Mach.getRepPermit(" +time+") AnRd2Wait="+AnReadyToWait);
	try{
        if( (running=isRunning())==true ){
          wait(time);
        } else {
		if(DEBUG>=2) System.out.println(""+getSum()+" Mach.getRepPermit:readyToWait? "+AnReadyToWait);
		if(AnReadyToWait) {wait(); running=true; }
	  }
        }catch(InterruptedException e){
      }
      if (!running) AnReadyToWait = true;
    }
*/

    /**
     * A method that checks whether Machine.run() method should continue with adding balls.
     * @param finished if the machine should pause (c==N)
     * @return whether Machine should dispose
     */

/*
    protected synchronized boolean getRunPermit(boolean finished){
        if(finished) setRunning(false);
        while(!running){
	    if(DEBUG>=2) System.out.println(""+getSum()+": Mach.getRunPermit:not running, will wait");
          try{
            processEvent(new StartStopEvent(getSum(),StartStopEvent.STOP));
            wait();
          }catch(InterruptedException e){
          }
	    if(DEBUG>=2) System.out.println(""+getSum()+": Mach.getRunPermit:finished waiting");
        }
        if(finished)clearCounter();
        return !dead;   //right now true
    }
*/

    /**
     * Sets n. Automatically pauses Machine.run() by calling setRunning(false) and clears Table, l, max.
     * @see n.
     */
    public synchronized void setn(int n){
	if(DEBUG>=1)System.out.println("Machine.setn( "+n+" )");
        if(isAlive()) stop();
        this.n=n;
        Table=new int[n+1][2];
        l=0;
        max=0;
        clearCounter();
        System.gc();
        notifyAll();
		processEvent(new AdjustmentEvent(Event.nADJUSTMENT,n));
    }
    public void setn(){ setn(n); }

    protected synchronized void addToTable(){
        double b=0.0;
        for(int i=0;i<n;i++)
	    if(random.nextFloat()<p) b+=0.5;
	                    else     b-=0.5;
        int k=(int)(b+ktox(n));   // same as xtok(b)
	if(Table[k][VALUE]==max)
            max++;
        Table[k][VALUE]++;
        Table[k][MODIF]++;
        l++;
        c++;
    }
    
    /**
     * To implement Runnable. 
     */
    public void run(){
		Thread thread=Thread.currentThread();
		if(DEBUG>=2) System.out.println(""+getSum()+": Mach.run/begin");
      
//      processEvent(new StartStopEvent(getSum(),StartStopEvent.START));
		while(thread == thisThread && c<N){
			addToTable();
		}
		if(DEBUG>=2) System.out.println(""+getSum()+": Mach.run/end");
		stop(); /* seems stupid, but otherwise components get isAlive()==true, when shouldn't. */
    }

    public void start(){
        System.out.println(""+getSum()+":  Machine.start()");
        if(isAlive()) { System.out.println(""+getSum()+":  ERROR: Mach.start() when Machine isAlive"); return;}
		if(Table==null) {System.out.println(""+getSum()+"Mach.start() on null table"); return; }
		if(c==N) clearCounter();
        thisThread = new Thread(this,"AnalysisMachine");
        thisThread.start();
        processEvent(new StartStopEvent(getSum(),StartStopEvent.START));
    }
    public void stop(){
        System.out.println(""+getSum()+":  Machine.stop()");
        thisThread = null;
        processEvent(new StartStopEvent(getSum(),StartStopEvent.STOP));
    }

/*       
    public synchronized void end(){
	dead=true;
      notifyAll();
    }
*/
	public boolean isAlive(){ return thisThread!=null && thisThread.isAlive(); }
	
	
	/****************************************************************************************************/
	/***********                      Event stuff               *****************************************/

	StartStopListener ssListener = null;
	AdjustmentListener adjustmentListener = null;
	public void addStartStopListener(StartStopListener l){ ssListener = EventMulticaster.add(ssListener, l); }
	public void removeStartStopListener(StartStopListener l){ ssListener = EventMulticaster.remove(ssListener, l); }
	public void addAdjustmentListener(AdjustmentListener l){ adjustmentListener = EventMulticaster.add(adjustmentListener, l); }
	public void removeAdjustmentListener(AdjustmentListener l){ adjustmentListener = EventMulticaster.remove(adjustmentListener, l); }
	
	public void processEvent(Event e){
		if(e instanceof  StartStopEvent){  processStartStopEvent(  (StartStopEvent)e ); return; }
		if(e instanceof AdjustmentEvent){ processAdjustmentEvent( (AdjustmentEvent)e ); return; }
	}
	
	public void processStartStopEvent(StartStopEvent e){
		if (ssListener != null) 
		 	switch(e.getType()){
				case Event.START : ssListener.machineStarted(e); break;
				case Event.STOP  : ssListener.machineStopped(e); break;
				default    : System.err.println("type UNKNOWN in Machine processStartStopEvent");
		}
	}
	public void processAdjustmentEvent(AdjustmentEvent e){
		if (adjustmentListener != null)
		 	switch(e.getType()){
				case Event.pADJUSTMENT : adjustmentListener.pChanged(e);break;
				case Event.nADJUSTMENT : adjustmentListener.nChanged(e);break;
				case Event.NADJUSTMENT : adjustmentListener.NChanged(e);break;
				case Event.CLEARCOUNTER: adjustmentListener.clearCounter(e);break;
				default    : System.err.println("type UNKNOWN in Machine processAdjustmentEvent");
			}
	}
}
