/*
 * @(#)Options.java
 *
 */
package galton.analysis;

import java.awt.*;
import java.awt.event.*;
import galton.analysis.Language;

/**
 * A dialogue window for changing distributions.
 *
 * @author Zbigniew Jedrzejewski-Szmek
 */

public class Options extends Frame implements ItemListener, ActionListener, WindowListener{
    
    public Options(String TITLE,ActionListener l,Language lang){

      this.lang = lang;

	dist=new DistributionVector();

	GridBagLayout gridbag=new GridBagLayout();
     	GridBagConstraints c=new GridBagConstraints();
	CheckboxGroup typeGroup=new CheckboxGroup();
       
	//	setFont(FONT);
	setLayout(gridbag);
	setBackground(Color.lightGray);
	setTitle(TITLE);
	
	//	c.weightx=1.0;
	c.ipadx=2; c.ipady=2;
	c.insets=new Insets(2,2,2,2);
	c.fill=GridBagConstraints.BOTH;
	
	c.gridwidth=GridBagConstraints.REMAINDER;
	name=new Choice();				//dist choice / name
	name.addItemListener(this);
	gridbag.setConstraints(name,c);
	add(name);
	
	c.gridwidth=GridBagConstraints.RELATIVE;	//type 0
	type[0]=new Checkbox(lang.getName(Language.BINOMIAL),typeGroup,false);
	type[0].addItemListener(this);
	gridbag.setConstraints(type[0],c);
	add(type[0]);
	
	c.gridwidth=GridBagConstraints.REMAINDER;
	visib = new Checkbox(lang.getName(Language.VISIBLE));
	visib.addItemListener(this);
	gridbag.setConstraints(visib,c);
	add(visib);
	
	c.gridwidth=GridBagConstraints.REMAINDER;	//type 1
	type[1]=new Checkbox(lang.getName(Language.NORMAL),typeGroup,false);
	type[1].addItemListener(this);
	gridbag.setConstraints(type[1],c);
	add(type[1]);
	
	c.gridwidth=GridBagConstraints.REMAINDER;	//type 2
	type[2]=new Checkbox(lang.getName(Language.POISSON),typeGroup,false);
	type[2].addItemListener(this);
	gridbag.setConstraints(type[2],c);
	add(type[2]);
	
	c.gridwidth=GridBagConstraints.RELATIVE;	//label 1
	label1=new Label("");
	label1.setAlignment(Label.RIGHT);
	gridbag.setConstraints(label1,c);
	add(label1);

	c.gridwidth=GridBagConstraints.REMAINDER;	//param 1
	param1=new TextField();
	param1.addActionListener(this);
	//	param1.addFocusListener(this);
	gridbag.setConstraints(param1,c);
	add(param1);
		
	c.gridwidth=GridBagConstraints.RELATIVE;	//label 2
	label2=new Label("");
	label2.setAlignment(Label.RIGHT);
	gridbag.setConstraints(label2,c);
	add(label2);
	
	c.gridwidth=GridBagConstraints.REMAINDER;	//param 2
	param2=new TextField();
	param2.addActionListener(this);
	//	param2.addFocusListener(this);
	gridbag.setConstraints(param2,c);
	add(param2);
	
	c.gridwidth=GridBagConstraints.RELATIVE;
	label3=new Label(lang.getName(Language.COLOR));
	label3.setAlignment(Label.RIGHT);
	gridbag.setConstraints(label3,c);
	add(label3);
	
	c.gridwidth=GridBagConstraints.REMAINDER;		//color Choice
	color=new Choice();
	color.addItemListener(this);
	try{
	    for(int i=0;i<13;i++)
		color.addItem(lang.getColorName(i));
	}catch(NullPointerException e){
	    System.out.println("Op.Options():"+e);
	}catch(IndexOutOfBoundsException e){
	    //not enough colors
	}
	color.select(lang.getColorName(Language.green)); //green
	gridbag.setConstraints(color,c);
	add(color);

        c.insets=new Insets(5,0,0,1);

	c.gridwidth=GridBagConstraints.RELATIVE;
	newButton = new Button(lang.getName(Language.NEW));                 	//button new
        newButton.addActionListener(this);
	gridbag.setConstraints(newButton,c);
	add(newButton);

	c.gridwidth=GridBagConstraints.REMAINDER;
	delButton = new Button(lang.getName(Language.DELETE));             	//button del
        delButton.addActionListener(this);
	gridbag.setConstraints(delButton,c);
	add(delButton);

        c.insets=new Insets(1,0,2,1);
	
	c.gridwidth=GridBagConstraints.RELATIVE;
	add(new Label(),c);
	
	c.gridwidth=GridBagConstraints.REMAINDER;
	delallButton = new Button(lang.getName(Language.DELALL));             	//button delall
        delallButton.addActionListener(this);
	gridbag.setConstraints(delallButton,c);
	add(delallButton);
    	
        c.insets=new Insets(5,0,2,1);
	
	c.gridwidth=GridBagConstraints.RELATIVE;
	OKButton=new Button(lang.getName(Language.OK));
        OKButton.addActionListener(this);
	OKButton.addActionListener(l);
	gridbag.setConstraints(OKButton,c);
	add(OKButton);
	
	c.gridwidth=GridBagConstraints.REMAINDER;
	helpButton=new Button(lang.getName(Language.HELP));
        helpButton.addActionListener(this);
	helpButton.addActionListener(l);
	gridbag.setConstraints(helpButton,c);
	add(helpButton);

        addWindowListener(this);
	pack();
	setResizable(false);
	
	if(DEBUG>=2){
	    for(int i=0;i<getComponentCount();i++)
		System.out.println("* "+i+". "+getComponent(i));
	    System.out.println("name:["+name.getItemCount()+"] "+name);   
	}
    }
    /* RUNTIME FUNCTIONS */
    private boolean parseParams(){ //return true if numbers are invalid
	try{
	    dist.distributionAt(index()).setParam(1,parseDouble(param1.getText()));
	}catch(NumberFormatException e){
	    System.err.println(""+e);
	    return true; //failure
	}
	try{
	    dist.distributionAt(index()).setParam(2,parseDouble(param2.getText()));
	}catch(NumberFormatException e){
	    System.err.println(""+e);
	    return true; //failure
	}
	return false; //successful
    }

    private synchronized int index(){
	if(dist.isEmpty()){
	    eventNew();
	    dist.distributionAt(0).setVisible(false);
	    return 0;
	}
	String title=name.getSelectedItem();
	int newIndex=0;
	while(dist.distributionAt(newIndex).getTitle()!=title)newIndex++;
	return newIndex;
    }
    
    private void eventNew(){
	String title=lang.getName(Language.DISTRIBUTION)+" "+dist.nextNumber();
        Distribution newDistrib=new Distribution(lang);
        newDistrib.setTitle(title);
	if(DEBUG>=3){
	    System.out.println("newDist:"+newDistrib);
	    System.out.println("name:["+name.getSize()+"] "+name);
	    System.out.println("adding: "+title);
	}
	name.addItem(title);
	name.select(title);
        dist.addDistribution(newDistrib);
	if(DEBUG>=3){
	    System.out.println("name:["+name.getItemCount()+"] "+name);   
	}
        update();
    }
    private void eventDel(){
        int i=index();
	if(DEBUG>=2) System.out.println("Trying to remove:"+i+" "+dist.distributionAt(i).getTitle());
	name.remove(dist.distributionAt(i).getTitle());
        dist.removeDistributionAt(i);
	if(i<name.getItemCount()) name.select(i);
			     else name.select(i-1);
    }
    private void eventDelAll(){
        dist.removeAll();
        name.removeAll();
        eventNew();
    }
    public void update(){
	Distribution cdist=dist.distributionAt(index());
	if(cdist==null){
	    if(DEBUG>=2) System.out.println("Op.update():dist[index()] is null!");
	    return;
	}
	if(type[cdist.getType()].getState()!=true) type[cdist.getType()].setState(true);
	if(visib.getState()!=cdist.isVisible()) visib.setState(cdist.isVisible());
	if(color.getSelectedItem()!=cdist.getColorName()) color.select(cdist.getColorName());
	param1.setText(""+cdist.getParam(1));
	param2.setText(""+cdist.getParam(2));
	if(label1.getText()!=cdist.getParamName(1)) label1.setText(cdist.getParamName(1));
	if(label2.getText()!=cdist.getParamName(2)) label2.setText(cdist.getParamName(2));
	param2.setEditable(cdist.useSecondParam());
    }

    /* UTILITY FUNCTIONS */
    public synchronized DistributionVector getDist(){
        return dist;
    }
    private double parseDouble(String s) throws NumberFormatException{
	java.lang.Double d = new java.lang.Double(s.startsWith("+")?
						  s.substring(1).replace(',','.'):
						  s.replace(',','.'));
	return d.doubleValue();
    }
    /* EVENT HANDLERS */
    public void actionPerformed(ActionEvent evt){
	if(DEBUG>=2)
	    System.out.println("Options.actionPerformed("+evt.getActionCommand()+")");
        if(evt.getActionCommand().equals(lang.getName(Language.DELETE)))      eventDel();
        if(evt.getActionCommand().equals(lang.getName(Language.DELALL)))   eventDelAll();
        if(evt.getActionCommand().equals(lang.getName(Language.NEW)))      eventNew();
        if(evt.getActionCommand().equals(lang.getName(Language.OK))){
            parseParams();
            dispose();
        }
        if(evt.getSource().equals(param1) || evt.getSource().equals(param2) ){
            parseParams();
            transferFocus();
        }
    }
    public void windowDeactivated(WindowEvent evt){
    }
    public void windowDeiconified(WindowEvent evt){
    }
    public void windowActivated(WindowEvent evt){
    }
    public void windowOpened(WindowEvent evt){
	if(packedSize!=null)this.setSize(packedSize);
	else    packedSize=this.getSize();
    }
    public void windowIconified(WindowEvent evt){
    }
    public void windowClosed(WindowEvent evt){
    }
    public void windowClosing(WindowEvent evt){
	OKButton.dispatchEvent(
        new ActionEvent(OKButton,ActionEvent.ACTION_FIRST,lang.getName(Language.OK)));
    }

    public void itemStateChanged(ItemEvent evt){
	if(DEBUG>=2)
	    System.out.println("Op.itemStateChanged("+evt+")");
        if(evt.getItemSelectable().equals(name)){
	    update();
	    return;
        }
        if(evt.getItemSelectable().equals(visib)){
	    dist.distributionAt(index()).setVisible(
		  evt.getStateChange()==ItemEvent.SELECTED);
            return;
	}
        if(evt.getItemSelectable().equals(color)){
	    dist.distributionAt(index()).setColor(color.getSelectedItem());
            return;
        }
	
	for(int i=0;i<3;i++){
	    if(evt.getItemSelectable().equals(type[i]) && 
	       (evt.getStateChange()==ItemEvent.SELECTED) ){
		dist.distributionAt(index()).setType(i);
		update();
	    }
	}
	
    }

    /* COMPONENTS AND VARIABLES */
    private DistributionVector dist;
    public Dimension packedSize;  // for a Netscape bug
    
    private Choice 		name;
    private Label 		label1,label2;
    private TextField		param1,param2;
    private Button 		newButton,delButton,delallButton;
    private Label 		label3;
    private Choice 		color;
    public  Button 		OKButton,helpButton;

    private Checkbox		visib;
    private CheckboxGroup	typeGroup;
    private Checkbox		type[]=new Checkbox[3];

    private static final int DEBUG=1;
    private Language lang;
}


