/*
 * @(#)Results.java
 */
package galton.analysis;

import java.awt.*;
import java.awt.event.*;
import java.text.*;
import java.util.*;
import galton.analysis.*;
import galton.common.ResultWindow;


/**
 * Manages snapshots, displaying them in ResultWindows.
 */
public class Results implements WindowListener{
    private Vector results;
    String TITLE="Results";    
    private Language lang;

    public Results(String TITLE,Language lang){
	this.TITLE=TITLE;
	this.results=new Vector();
      this.lang = lang;
    }

    public void display(Machine machine,DistributionVector dist){

	String text1=machineToString(machine)+"\n"+columnToString(machine,dist,HEADER);
      String text2="";
	String text3="";
	for(int k=0;k<=machine.n;k++)
	    text2=text2+columnToString(machine,dist,k);
	for(int i=0;i<dist.getSize();i++)
	    text3=text3+dist.distributionAt(i).toString()+"\n";
	text2+="\n";


      new ResultWindow(text1+text2+text3,generateTitle(),this).show();
	if(DEBUG>=2) System.out.println("results.size() "+results.size());
    }
    
    public String generateTitle(){
      Date date=new Date();
      return TITLE+" "+
                  DateFormat.getDateInstance(DateFormat.LONG).format(date)+", "+
                  DateFormat.getTimeInstance(DateFormat.MEDIUM).format(date);
    }

    public String machineToString(Machine machine){
      NumberFormat nf = NumberFormat.getInstance();
      nf.setMaximumFractionDigits(8);
      nf.setGroupingUsed(false);
	machine.makeInfo();
	String res = lang.getName(Language.nSTRING)+" = "+nf.format(machine.n)+"\n";
	res+=lang.getName(Language.SUM)+" : "+nf.format(machine.getSum())+"\n";
	for(int i=0;i<4;i++) res+=lang.getName(Language.INFO+i)+" = "+nf.format(machine.getInfo(i))+"\n";
	return res;
    }

    public String columnToString(Machine machine, DistributionVector dvector,int k){
	int dcolumns=dvector.getSize();	//the number of active distributions
	if(k==HEADER){
	    String ret=lang.getName(Language.kSTRING)+lang.getTabulator()+lang.getName(Language.NSTRING)+lang.getTabulator()
            +lang.getName(Language.xSTRING)+lang.getTabulator()+lang.getName(Language.NSTRING)+"(%)";
	    for(int i=0;i<dcolumns;i++) 
		  if (dvector.distributionAt(i)!=null) ret+=lang.getTabulator()+dvector.distributionAt(i).ID();
	    return ret.concat("\n");
	} else {
            NumberFormat nf = NumberFormat.getInstance();
            nf.setMaximumFractionDigits(5);
            nf.setGroupingUsed(false);
            String ret=nf.format(k)+lang.getTabulator()+nf.format(machine.getValue(k))+
		lang.getTabulator()+nf.format(machine.ktox(k))+lang.getTabulator();
	    ret+=nf.format((double)machine.getValue(k)/(double)machine.getSum());
	    for(int i=0;i<dcolumns;i++) ret+=lang.getTabulator()+nf.format(dvector.distributionAt(i).getValue((double)k,machine.ktox(k)));
	    return ret.concat("\n");
	}
    }


    private static final String 	DIVISOR="-";
/*
    private static final String 	TAB="\t\t";
*/    
    private static final int 	      HEADER=-1;
    private static final int 		DEBUG = 0;


    public void windowClosing(WindowEvent evt){
	if(DEBUG>=1) System.out.print("windowClosing");
	Window w=evt.getWindow();
	if(DEBUG>=2) System.out.println(" : "+w);
	w.dispose();
	if(DEBUG>=2) System.out.println(" : "+w);
	w=null;
	if(DEBUG>=2) System.out.println(" : "+w);
    }
    public void windowDeactivated(WindowEvent evt){
	if(DEBUG>=3) System.out.println("windowDeactivated");
    }
    public void windowDeiconified(WindowEvent evt){
	if(DEBUG>=3) System.out.println("windowDeiconified");
    }
    public void windowActivated(WindowEvent evt){
	if(DEBUG>=3) System.out.println("windowActivated");
    }
    public void windowOpened(WindowEvent evt){
	if(DEBUG>=3) System.out.println("windowOpened");
    }
    public void windowIconified(WindowEvent evt){
	if(DEBUG>=3) System.out.println("windowIconified");
    }
    public void windowClosed(WindowEvent evt){
	if(DEBUG>=3) System.out.println("windowClosed");	      
    }
}