package galton.event;

import galton.event.*;
import java.util.EventListener;

/**
 * A class which implements multi-cast event dispatching for galton.event.Event and subclasses. 
 * The idea and most of the code comes from AWTEventMulticaster.
 */
public class EventMulticaster 
implements StartStopListener, AdjustmentListener {
    protected final EventListener a, b;

    //**********************************************************************************************[ add & remove ]
    public static StartStopListener add(StartStopListener a,StartStopListener b){ return (StartStopListener) addInternal(a,b); }
    public static StartStopListener remove(StartStopListener l,StartStopListener old){ 
        return (StartStopListener) removeInternal(l,old); 
    }
    public static AdjustmentListener add(AdjustmentListener a,AdjustmentListener b){ return (AdjustmentListener) addInternal(a,b); }
    public static AdjustmentListener remove(AdjustmentListener l,AdjustmentListener old){ 
        return (AdjustmentListener) removeInternal(l,old); 
    }


    //**********************************************************************************************[ action methods ]
	public void machineStopped(StartStopEvent e){ 
        ((StartStopListener)a).machineStopped(e); 
        ((StartStopListener)b).machineStopped(e);
    }
    public void machineStarted(StartStopEvent e){ 
        ((StartStopListener)a).machineStarted(e); 
        ((StartStopListener)b).machineStarted(e);
    }

    public void pChanged(AdjustmentEvent e){
        ((AdjustmentListener)a).pChanged(e); 
        ((AdjustmentListener)b).pChanged(e);
    }
    public void nChanged(AdjustmentEvent e){
        ((AdjustmentListener)a).nChanged(e); 
        ((AdjustmentListener)b).nChanged(e);
    }
    public void NChanged(AdjustmentEvent e){
        ((AdjustmentListener)a).NChanged(e); 
        ((AdjustmentListener)b).NChanged(e);
    }
    public void clearCounter(AdjustmentEvent e){
        ((AdjustmentListener)a).clearCounter(e); 
        ((AdjustmentListener)b).clearCounter(e);
    }	
	
	
    //**********************************************************************************************[ constructors ]
    protected EventMulticaster(EventListener a, EventListener b) {
		this.a = a; this.b = b;
    }

    //**********************************************************************************************[ internal methods ]
    protected EventListener remove(EventListener oldl) {
		if (oldl == a)  return b;
		if (oldl == b)  return a;
		EventListener a2 = removeInternal(a, oldl);
		EventListener b2 = removeInternal(b, oldl);
		if (a2 == a && b2 == b)
		    return this;	// it's not here
		return addInternal(a2, b2);
    }

    protected static EventListener addInternal(EventListener a, EventListener b) {
		if (a == null)  return b;
		if (b == null)  return a;
		return new EventMulticaster(a, b);
    }

    protected static EventListener removeInternal(EventListener l, EventListener oldl) {
		if (l == oldl || l == null) return null;
		if (l instanceof EventMulticaster) return ((EventMulticaster)l).remove(oldl);
		return l;		// it's not here
    }      
}